@@ -277,17 +277,29 @@ GtkWidget *create_scan_window() {
     
     // create the labels and other description stuff which will be on the right hand side of the screen
     strcpy(appdata->EAN_text, "0000000000000");
-    appdata->EANlabel = gtk_label_new (appdata->EAN_text);
+    //appdata->EANlabel = gtk_label_new (appdata->EAN_text);
+    appdata->EANlabel = hildon_entry_new(HILDON_SIZE_AUTO);
+    hildon_entry_set_text(GTK_ENTRY(appdata->EANlabel), appdata->EAN_text);
+    gtk_entry_set_editable(GTK_ENTRY(appdata->EANlabel), FALSE);
 
     strcpy(appdata->title_text, "unknown");
-    appdata->titlelabel = gtk_label_new (appdata->title_text);
-    gtk_label_set_line_wrap(appdata->titlelabel, TRUE); // set line wrapping so we can see all the output
+    //gtk_label_set_line_wrap(appdata->titlelabel, TRUE); // set line wrapping so we can see all the output
+    //gtk_label_set_line_wrap(appdata->titlelabel, TRUE); // set line wrapping so we can see all the output
+    appdata->titlelabel = hildon_entry_new(HILDON_SIZE_AUTO);
+    hildon_entry_set_text(GTK_ENTRY (appdata->titlelabel), appdata->title_text);
+    gtk_entry_set_editable(GTK_ENTRY (appdata->titlelabel), FALSE);
 
     strcpy(appdata->author_text, "unknown");
-    appdata->authorlabel = gtk_label_new (appdata->author_text);
+    //appdata->authorlabel = gtk_label_new (appdata->author_text);
+    appdata->authorlabel = hildon_entry_new(HILDON_SIZE_AUTO);
+    hildon_entry_set_text(GTK_ENTRY (appdata->authorlabel), appdata->author_text);
+    gtk_entry_set_editable(GTK_ENTRY (appdata->authorlabel), FALSE);
 
     strcpy(appdata->price_text, "unknown");
-    appdata->pricelabel = gtk_label_new (appdata->price_text);
+    //appdata->pricelabel = gtk_label_new (appdata->price_text);
+    appdata->pricelabel = hildon_entry_new(HILDON_SIZE_AUTO);
+    hildon_entry_set_text(GTK_ENTRY (appdata->pricelabel), appdata->price_text);
+    gtk_entry_set_editable(GTK_ENTRY (appdata->pricelabel), FALSE);
 
     // pack the labels, etc
     gtk_box_pack_start (GTK_BOX (vbox_right), appdata->EANlabel, TRUE, TRUE, 0);
@@ -323,10 +335,14 @@ GtkWidget *create_scan_window() {
 }
 
 void update_scan_window() {
-    gtk_label_set_text ((GtkLabel *)appdata->EANlabel, appdata->EAN_text);
-    gtk_label_set_text((GtkLabel *)appdata->titlelabel, appdata->title_text);
-    gtk_label_set_text((GtkLabel *)appdata->authorlabel, appdata->author_text);
-    gtk_label_set_text((GtkLabel *)appdata->pricelabel, appdata->price_text);
+    //gtk_label_set_text ((GtkLabel *)appdata->EANlabel, appdata->EAN_text);
+    //gtk_label_set_text((GtkLabel *)appdata->titlelabel, appdata->title_text);
+    //gtk_label_set_text((GtkLabel *)appdata->authorlabel, appdata->author_text);
+    //gtk_label_set_text((GtkLabel *)appdata->pricelabel, appdata->price_text);
+    hildon_entry_set_text(GTK_ENTRY(appdata->EANlabel), appdata->EAN_text);
+    hildon_entry_set_text(GTK_ENTRY(appdata->titlelabel), appdata->title_text);
+    hildon_entry_set_text(GTK_ENTRY(appdata->authorlabel), appdata->author_text);
+    hildon_entry_set_text(GTK_ENTRY(appdata->pricelabel), appdata->price_text);
 }
 
 
