@@ -32,9 +32,13 @@
 #include <libosso-abook/osso-abook-util.h>
 #endif /* FREMANTLE */
 
+#include <libmodest-dbus-client/libmodest-dbus-client.h>
+
 //#include <hildon/hildon-helper.h>
 //#include <hildon/hildon-defines.h>
-//#include <hildon-mime.h>
+
+#include <hildon-mime.h>
+
 #include "web.h"
 
 #ifdef FREMANTLE
@@ -1782,13 +1786,9 @@ int parse_mixed_barcode_data(char *data)
 
     // the assumption here is that the barcode is definitely not an EAN
 
-
     // this link used for identifiers and ideas.
     // http://code.google.com/p/zxing/wiki/BarcodeContents
 
-
-
-
     printf("Mixed barcode: %s\n", data);
 
     strcpy(appdata->EAN_text, data);
@@ -1798,13 +1798,10 @@ int parse_mixed_barcode_data(char *data)
     update_scan_window();
 
 
-
-
-
-
     // check for a vCard
-    urlp = strstr(data, "BEGIN:VCARD");
-    if (urlp){
+    //urlp = strstr(data, "BEGIN:VCARD");
+    if (strncasecmp(data, "BEGIN:VCARD", 11)==0){
+        urlp = data;
         // we assume there's nothing of interest on the end
         printf("vCard!\n");
         banner = hildon_banner_show_information (appdata->window, NULL, "vCard");
@@ -1816,50 +1813,47 @@ int parse_mixed_barcode_data(char *data)
     }
 
 
-
-
-
-
-
-
-    
     // see if this contains an URL
     // we should check for this last, as there may be an URL in e.g. a VCard
     
     // check for the URL: header
-    urlp = strstr(data, "URL:");
-    if (urlp){
+    //urlp = strstr(data, "URL:");
+    if (strncasecmp(data, "URL:", 4)==0){
         urlp = urlp+4; // skip the URL: business
         
         printf("URL!\n");
         banner = hildon_banner_show_information (appdata->window, NULL, "URL");
         hildon_banner_set_timeout (HILDON_BANNER (banner), 3000);
         // we assume there's nothing of interest on the end
-        openURL(urlp); // open the url
+        //openURL(urlp); // open the url
+        openGeneric(data, "Open URL: %s?");
         return TRUE;
     }
     
     // check for the URLTO: header
-    urlp = strstr(data, "URLTO:");
-    if (urlp){
-        urlp = urlp+6; // skip the URLTO: business
+    //urlp = strstr(data, "URLTO:");
+    if (strncasecmp(data, "URLTO:", 6)==0){
+        urlp = data+6; // skip the URLTO: business
         
         printf("URL!\n");
         banner = hildon_banner_show_information (appdata->window, NULL, "URL");
         hildon_banner_set_timeout (HILDON_BANNER (banner), 3000);
         // we assume there's nothing of interest on the end
-        openURL(urlp); // open the url
+        //openURL(urlp); // open the url
+        openGeneric(data, "Open URL: %s?");
         return TRUE;
     }
     
     // check for a plain http://
+    // might want to check for odd cases, though it *should* be lower case
     urlp = strstr(data, "http://");
     if (urlp){        
         printf("URL!\n");
         banner = hildon_banner_show_information (appdata->window, NULL, "URL");
         hildon_banner_set_timeout (HILDON_BANNER (banner), 3000);
         // we assume there's nothing of interest on the end
-        openURL(urlp); // open the url
+        //openURL(urlp); // open the url
+        openGeneric(data, "Open URL: %s?");
         return TRUE;
     }
     
@@ -1869,31 +1863,31 @@ int parse_mixed_barcode_data(char *data)
     // if it's QR code, check the iMode stuff
     // http://www.nttdocomo.co.jp/english/service/imode/make/content/barcode/function/content/index.html
     
-    if (strncmp(data, "CNTS:", 5)){
+    if (strncasecmp(data, "CNTS:", 5)==0){
         printf("i-mode content marker\n");
-        banner = hildon_banner_show_information (appdata->window, NULL, "i-mode content marker");
+        banner = hildon_banner_show_informationf (appdata->window, NULL, "i-mode content marker: %s", data);
         hildon_banner_set_timeout (HILDON_BANNER (banner), 3000);
         //type = data[10:14]
         //lng = data[18:22]
         //body = data[27:27+lng+1]
         return TRUE;
-    }else if (strncmp(data, "MECARD:", 7)){ //#http://www.nttdocomo.co.jp/english/service/imode/make/content/barcode/function/application/common/index.html
+    }else if (strncasecmp(data, "MECARD:", 7)==0){ //#http://www.nttdocomo.co.jp/english/service/imode/make/content/barcode/function/application/common/index.html
         printf("Phone book registration message\n");
         banner = hildon_banner_show_information (appdata->window, NULL, "Phone book registration message");
         hildon_banner_set_timeout (HILDON_BANNER (banner), 3000);
         return TRUE;
-    }else if (strncmp(data, "MATMSG:", 7)){
+    }else if (strncasecmp(data, "MATMSG:", 7)==0){
         printf("Email sync message\n");
         banner = hildon_banner_show_information (appdata->window, NULL, "Email sync message");
         hildon_banner_set_timeout (HILDON_BANNER (banner), 3000);
         return TRUE;
-    }else if (strncmp(data, "MEBKM:", 6)){
+    }else if (strncasecmp(data, "MEBKM:", 6)==0){
         printf("Bookmark registration message\n");
         // we could try parsing this out and asking the user if they want to save it as a bookmark, or just open it in the browser
         banner = hildon_banner_show_information (appdata->window, NULL, "Bookmark registration message");
         hildon_banner_set_timeout (HILDON_BANNER (banner), 3000);
         return TRUE;
-    }else if (strncmp(data,"LAPL:", 5)){
+    }else if (strncasecmp(data,"LAPL:", 5)==0){
         printf("i-appli sync message\n");
         banner = hildon_banner_show_information (appdata->window, NULL, "i-appli sync message");
         hildon_banner_set_timeout (HILDON_BANNER (banner), 3000);
@@ -1904,26 +1898,30 @@ int parse_mixed_barcode_data(char *data)
     
     
     // check for an email address...
-    if (strncmp(data, "mailto:", 7)){        
+    if (strncasecmp(data, "mailto:", 7)==0){        
         printf("Email address\n");
         banner = hildon_banner_show_information (appdata->window, NULL, "Email address");
         hildon_banner_set_timeout (HILDON_BANNER (banner), 3000);
+        //openEmail((char*)(data+7)); // just pass the remainder on
+        openGeneric(data, "Open Email: %s?");
         return TRUE;
     }
 
 
     // check for a telephone number...
-    if (strncmp(data, "tel:", 4)){      
+    if (strncasecmp(data, "tel:", 4)==0){      
         printf("Telephone number\n");
         banner = hildon_banner_show_information (appdata->window, NULL, "Telephone number");
         hildon_banner_set_timeout (HILDON_BANNER (banner), 3000);
+        //openTel(data);
+        openGeneric(data, "Open Telephone Number: %s?");
         return TRUE;
     }
     
 
 
     // check for a BIZCARD
-    if (strncmp(data, "BIZCARD:", 8)){      
+    if (strncasecmp(data, "BIZCARD:", 8)==0){      
         printf("BIZCARD\n");
         banner = hildon_banner_show_information (appdata->window, NULL, "BIZCARD");
         hildon_banner_set_timeout (HILDON_BANNER (banner), 3000);
@@ -1932,47 +1930,52 @@ int parse_mixed_barcode_data(char *data)
 
 
     // check for an SMS
-    if (strncmp(data, "sms:", 4)){      
+    if (strncasecmp(data, "sms:", 4)==0){      
         printf("SMS\n");
         banner = hildon_banner_show_information (appdata->window, NULL, "SMS");
         hildon_banner_set_timeout (HILDON_BANNER (banner), 3000);
+        openGeneric(data, "Open SMS: %s?");
         return TRUE;
     }
-    if (strncmp(data, "smsto:", 6)){
+    if (strncasecmp(data, "smsto:", 6)==0){
         printf("SMS\n");
         banner = hildon_banner_show_information (appdata->window, NULL, "SMS");
         hildon_banner_set_timeout (HILDON_BANNER (banner), 3000);
+        openGeneric(data, "Open SMS: %s?");
         return TRUE;
     }
 
 
     // check for an MMS
-    if (strncmp(data, "mms:", 4)){      
+    if (strncmp(data, "mms:", 4)==0){      
         printf("MMS\n");
         banner = hildon_banner_show_information (appdata->window, NULL, "MMS");
         hildon_banner_set_timeout (HILDON_BANNER (banner), 3000);
+        openGeneric(data, "Open MMS: %s?");
         return TRUE;
     }
-    if (strncmp(data, "mmsto:", 6)){
+    if (strncasecmp(data, "mmsto:", 6)==0){
         printf("MMS\n");
         banner = hildon_banner_show_information (appdata->window, NULL, "MMS");
         hildon_banner_set_timeout (HILDON_BANNER (banner), 3000);
+        openGeneric(data, "Open MMS: %s?");
         return TRUE;
     }
     
 
     // check for geographic information
-    if (strncmp(data, "geo:", 4)){      
+    if (strncasecmp(data, "geo:", 4)==0){      
         printf("Geographical information\n");
         banner = hildon_banner_show_information (appdata->window, NULL, "Geographical information");
         hildon_banner_set_timeout (HILDON_BANNER (banner), 3000);
+        openGeneric(data, "Show Geo Data: %s?");
         return TRUE;
     }
 
 
 
     // Android specific - link to app in Market
-    if (strncmp(data, "market:", 7)){      
+    if (strncasecmp(data, "market:", 7)==0){      
         printf("Android Market link\n");
         banner = hildon_banner_show_information (appdata->window, NULL, "Android Market link");
         hildon_banner_set_timeout (HILDON_BANNER (banner), 3000);
@@ -1982,7 +1985,7 @@ int parse_mixed_barcode_data(char *data)
 
 
     // iPhone rumoured - link to YouTube
-    if (strncmp(data, "youtube:", 8)){      
+    if (strncasecmp(data, "youtube:", 8)==0){      
         printf("Link to YouTube\n");
         banner = hildon_banner_show_information (appdata->window, NULL, "Link to YouTube");
         hildon_banner_set_timeout (HILDON_BANNER (banner), 3000);
@@ -1990,7 +1993,26 @@ int parse_mixed_barcode_data(char *data)
     }
 
 
-    // youtube://
+/*
+    // Let's try something Maemo specific - link to an app. E.g. and install file in a barcode
+    // we should probably parse the file and let the user know what it's for
+    // then we can also save the install file somewhere, then pass it onto app mngr
+    // so, what do we name the file? Use something from the contents (so it's understandable)
+    // or give it a unique (and not very useful) name?
+    if (strncasecmp(data, "[install]", 9)==0){ 
+        FILE *install_file;
+        char filename;
+        
+        install_file = fopen(const char *path, const char *mode);
+        printf("Maemo install file\n");
+        banner = hildon_banner_show_information (appdata->window, NULL, "Maemo install file");
+        hildon_banner_set_timeout (HILDON_BANNER (banner), 3000);
+        
+        openGeneric(filename, "Open install file: %s?");
+        return TRUE;
+    }
+*/
+
     
     // iCal
     
@@ -2211,8 +2233,154 @@ int handle_EAN8(char *data)
 
 
 
+// ===============================================================================================
+// Handlers for our different payload types
+// ===============================================================================================
 
 
+void openGeneric(char *uri1, char *question)
+{
+    GtkWidget *dialog;
+    gchar *uri = g_strdup(uri1);
+
+    dialog = gtk_message_dialog_new(GTK_WINDOW(appdata->window),
+            GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+            GTK_MESSAGE_QUESTION,
+            GTK_BUTTONS_YES_NO,
+            question, uri);
+            
+    g_signal_connect(dialog, "response", G_CALLBACK (openGeneric_response_cb), uri);
+    gtk_window_set_transient_for  (GTK_WINDOW (dialog), GTK_WINDOW (appdata->window));
+    gtk_widget_show_all (dialog);
+    
+}
+
+
+void openGeneric_response_cb (GtkDialog *dialog, gint response, gpointer user_data)
+{
+    char *uri = (char*)user_data;
+    GError* error;
+    HildonURIAction* action;
+    
+    gtk_widget_destroy(dialog);
+    
+    switch (response) {
+        case GTK_RESPONSE_YES:
+            action = hildon_uri_get_default_action_by_uri(uri, &error);
+            hildon_uri_open(uri, action, &error); //hildon_uri_open (uri, HILDON_URI_ACTION_NORMAL, error);
+            break;
+        default:
+            break; // dialog was cancelled, so do nothing
+    }
+    g_free(uri);
+}
+
+
+
+
+/*
+        dbus-send --dest=com.nokia.CallUI --type=method_call /com/nokia/CallUI com.nokia.CallUI.CreateCall string:"tel:0123456"
+        
+        msg = dbus_message_new_method_call("com.nokia.CallUI", // target for the method call
+         "/com/nokia/CallUI", // object to call on
+         "com.nokia.CallUI.CreateCall", // interface to call on
+         "string:\"tel:0123456\""); // method name
+*/
+
+void openTel(char *uri)
+{
+    GtkWidget *dialog;
+    gchar *tel = g_strdup(uri);
+    
+    printf("Opening Phone app to dial: %s\n", tel);
+
+    dialog = gtk_message_dialog_new(GTK_WINDOW(appdata->window),
+            GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+            GTK_MESSAGE_QUESTION,
+            GTK_BUTTONS_YES_NO,
+            "Dial number: %s", tel);
+            
+    g_signal_connect(dialog, "response", G_CALLBACK (openTel_response_cb), tel);
+    gtk_window_set_transient_for  (GTK_WINDOW (dialog), GTK_WINDOW (appdata->window));
+    gtk_widget_show_all (dialog);
+    
+}
+
+
+// http://efforts.embedded.ufcg.edu.br/maemo/
+void openTel_response_cb (GtkDialog *dialog, gint response, gpointer user_data)
+{
+    char *tel = (char*)user_data;
+    GError* error;
+    HildonURIAction* action;
+    
+    gtk_widget_destroy(dialog);
+    
+    switch (response) {
+        case GTK_RESPONSE_YES:
+            action = hildon_uri_get_default_action_by_uri(tel, &error);
+            hildon_uri_open(tel, action, &error); //hildon_uri_open (uri, HILDON_URI_ACTION_NORMAL, error);
+            break;
+        default:
+            break; // dialog was cancelled, so do nothing
+    }
+    g_free(tel);
+}
+
+
+
+
+
+
+void openEmail(char *address)
+{
+    GtkWidget *dialog;
+    gchar *uri = g_strdup(address);
+    
+    printf("Opening Email client to send to: %s\n", uri);
+
+    dialog = gtk_message_dialog_new(GTK_WINDOW(appdata->window),
+            GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+            GTK_MESSAGE_QUESTION,
+            GTK_BUTTONS_YES_NO,
+            "Send email to: %s", uri);
+            
+    g_signal_connect(dialog, "response", G_CALLBACK (openEmail_response_cb), uri);
+    gtk_window_set_transient_for  (GTK_WINDOW (dialog), GTK_WINDOW (appdata->window));
+    gtk_widget_show_all (dialog);
+    
+}
+
+
+// http://wiki.maemo.org/Documentation/Maemo_5_Developer_Guide/Using_Data_Sharing/SendVia_Functionality
+void openEmail_response_cb (GtkDialog *dialog, gint response, gpointer user_data)
+{
+    char *uri = (char*)user_data;
+    gboolean result = TRUE;
+        
+    gtk_widget_destroy(dialog);
+    
+    switch (response) {
+        case GTK_RESPONSE_YES:
+            result = libmodest_dbus_client_compose_mail(appdata->osso, /*osso_context_t*/
+                uri, /*to*/
+                NULL, /*cc*/
+                NULL, /*bcc*/
+                NULL, /*body*/
+                NULL, /*subject*/
+                NULL /*attachments*/);
+
+            break;
+        default:
+            break; // dialog was cancelled, so do nothing
+    }
+    if(result)
+        printf("Opened email... leaving.\n");
+    else
+        printf("Failed to open email... leaving.\n");
+
+    g_free(uri);
+}
 
 
 // good lad qwerty12 (http://lists.maemo.org/pipermail/maemo-developers/2009-September/020791.html)
